﻿using System;
using System.Collections.Generic;
using UnityEngine;

namespace PathFinding
{
    public class GenerateGrid
    {
        private static GenerateGrid _instance = null;
        private List<GameObject> planes;
        public List<GridTracker> Grids;
        private float _nodeDistance = 1.5f;
        public List<Vector3> Raycasts;

        public static GenerateGrid GetInstance()
        {
            if (_instance == null)
                _instance = new GenerateGrid();

            return _instance;
        }

        public void disableClosestNode(Vector3 pos)
        {
            Node n = findClosestNode(pos);
            n.disable();
        }

        public GenerateGrid()
        {
            planes = new List<GameObject>();
            Grids = new List<GridTracker>();
            _nodeDistance = GameObject.FindGameObjectWithTag("GameController").GetComponent<GameController>().NodeDistance;


            foreach (GameObject obj in GameObject.FindGameObjectsWithTag("Generate_Grid"))
                placeNodes(obj);
            createGraphs();
            connectGraphs();

            foreach (GameObject obj in GameObject.FindGameObjectsWithTag("Generate_Grid"))
                obj.SetActive(false);
        }


        private void placeNodes(GameObject obj)
        {
            Vector3 dimensions = obj.GetComponent<MeshCollider>().bounds.size;
            GridTracker gt = new GridTracker(obj);

            int widthCount = (int)(dimensions.z / _nodeDistance) + 1;
            int lengthCount = (int)(dimensions.x / _nodeDistance) + 1;
            Vector3 startPos = obj.transform.position;

            gt.width = widthCount;
            gt.height = lengthCount;

            startPos.x -= dimensions.x / 2;
            startPos.z -= dimensions.z / 2;
            startPos.x += (dimensions.x % _nodeDistance) / 2;
            startPos.z += (dimensions.z % _nodeDistance) / 2;

            for (int i = 0; i < widthCount; i++)
            {
                for (int j = 0; j < lengthCount; j++)
                {
                    float xPosition = startPos.x + (j * _nodeDistance);
                    float zPosition = startPos.z + (i * _nodeDistance);

                    gt.nodes.Add(new Node(((i * lengthCount) + j), xPosition, zPosition));
                }
            }
            Grids.Add(gt);
        }

        private void createGraphs()
        {
            foreach (GridTracker gt in Grids)
            {
                foreach (Node n in gt.nodes)
                {
                    
                        if ((n.ID + 1) % gt.height != 0)
                        {
                            if (n.ID - gt.height > 0)
                                new Line(n, gt.nodes[n.ID + 1 - gt.height]);
                            new Line(n, gt.nodes[n.ID + 1]);
                            if (n.ID + gt.height < gt.nodes.Count - 1)
                                new Line(n, gt.nodes[n.ID + 1 + gt.height]);
                        }

                        if ((n.ID - gt.height) > 0)
                            new Line(n, gt.nodes[n.ID - gt.height]);
                        if ((n.ID + gt.height) < gt.nodes.Count - 1)
                            new Line(n, gt.nodes[n.ID + gt.height]);
                        
                        if (n.ID % gt.height != 0)
                        {
                            if (n.ID - gt.height > 0)
                                new Line(n, gt.nodes[n.ID - 1 - gt.height]);
                            new Line(n, gt.nodes[n.ID - 1]);
                            if (n.ID + gt.height < gt.nodes.Count - 1)
                                new Line(n, gt.nodes[n.ID - 1 + gt.height]);
                        }
                    
                    
                }
            }
        }
        private void connectGraphs()
        {
            foreach(GridTracker gt in Grids)
            {
                Vector3 diff = (gt.nodes[0].Vector3(0) - gt.nodes[gt.height].Vector3(0)) / 2;
                for (int i = 0; i < gt.height; i++)
                {
                    Node n = findClosestNode(gt.nodes[i].Vector3(0) + diff);
                    if (n != gt.nodes[i])
                    {
                        new Line(n, gt.nodes[i]);
                        new Line(gt.nodes[i], n);
                    }
                    else
                        break;
                }

                diff = (gt.nodes[(gt.width - 1) * gt.height].Vector3(0) - gt.nodes[(gt.width - 2) * gt.height].Vector3(0)) / 2;
                for (int i = 0; i < gt.height; i++)
                {
                    Node n = findClosestNode(gt.nodes[((gt.width - 1) * gt.height) + i].Vector3(0));
                    if (n != gt.nodes[((gt.width - 1) * gt.height) + i])
                    {
                        new Line(n, gt.nodes[((gt.width - 1) * gt.height) + i]);
                        new Line(gt.nodes[((gt.width - 1) * gt.height) + i], n);
                    }
                    else
                        break;
                }

                diff = (gt.nodes[0].Vector3(0) - gt.nodes[1].Vector3(0)) / 2;
                for (int i = 0; i < gt.width; i++)
                {
                    Node n = findClosestNode(gt.nodes[i * gt.height].Vector3(0) + diff);
                    if (n != gt.nodes[i * gt.height])
                    {
                        new Line(n, gt.nodes[i * gt.height]);
                        new Line(gt.nodes[i * gt.height], n);
                    }
                    else
                        break;
                }

                diff = (gt.nodes[gt.height - 1].Vector3(0) - gt.nodes[gt.height - 2].Vector3(0)) / 2;
                for (int i = 0; i < gt.width; i++)
                {
                    Node n = findClosestNode(gt.nodes[i * gt.height + gt.height - 1].Vector3(0) + diff);
                    if (n != gt.nodes[i * gt.height + gt.height - 1])
                    {
                        new Line(n, gt.nodes[i * gt.height + gt.height - 1]);
                        new Line(gt.nodes[i * gt.height + gt.height - 1], n);
                    }
                    else
                        break;
                }
            }
        }

        /// <summary>
        /// Determine the closest node relative to a given position
        /// </summary>
        public Node findClosestNode(Vector3 pos)
        {
            Node selectedNode = null;
            float distance = -1;

            foreach(GridTracker gt in Grids)
            {
                Node n = findClosestNode(gt, pos);
                if (Vector3.Distance(n.Vector3(pos.y), pos) < distance || distance == -1)
                {
                    selectedNode = n;
                    distance = Vector3.Distance(n.Vector3(pos.y), pos);
                }
            }

            return selectedNode;
        }
        private Node findClosestNode(GridTracker gt, Vector3 pos)
        {
            bool searching = true;
            int currID = 0;
            Node selectedNode = null;

            while (searching)
            {
                float dis1 = -1, dis2 = -1, dis3 = -1;

                if (currID - gt.height > 0)
                    dis1 = Vector3.Distance(gt.nodes[currID - gt.height].Vector3(pos.y), pos);
                dis2 = Vector3.Distance(gt.nodes[currID].Vector3(pos.y), pos);
                if (currID + gt.height < (gt.nodes.Count - 1))
                    dis3 = Vector3.Distance(gt.nodes[currID + gt.height].Vector3(pos.y), pos);

                if (dis1 != -1 && dis1 < dis2)
                    currID -= gt.height;
                else if (dis3 != -1 && dis3 < dis2)
                    currID += gt.height;
                else
                {
                    float distance = -1;
                    for (int i = 0; i < gt.height; i++)
                    {
                        if (Vector3.Distance(gt.nodes[currID + i].Vector3(pos.y), pos) < distance || distance == -1)
                        {
                            selectedNode = gt.nodes[currID + i];
                            distance = Vector3.Distance(gt.nodes[currID + i].Vector3(pos.y), pos);
                        }
                    }
                    searching = false;
                }
            }
            return selectedNode;
        }
        private GridTracker findGridTracker(GameObject obj)
        {
            foreach (GridTracker gt in Grids)
                if (gt.ConnectedObject == obj)
                    return gt;
            return null;
        }
    }
}

