﻿using System.Collections.Generic;
using UnityEngine;

namespace PathFinding
{
    public class GeneratePath
    {
        private GenerateGrid _gg;
        private Node startNode, endNode;
        private List<Line> path;
        private List<Node> leaves;

        public GeneratePath(Vector3 start, Vector3 end)
        {
            _gg = GenerateGrid.GetInstance();
            startNode = _gg.findClosestNode(start);
            endNode = _gg.findClosestNode(end);
            path = new List<Line>();
            leaves = new List<Node>();
        }

        public List<Node> GetPath()
        {
            return calculatePath();
        }

        private List<Node> calculatePath()
        {
            bool searching = true;
            leaves.Add(startNode);
            startNode.stepCount = 0;
            startNode.DistanceTracker = calculateNodeDistance(startNode);
            List<Node> path = new List<Node>();

            while (searching)
            {
                Node n = getShortestLeave();
                foreach (Line l in n.Connections)
                {
                    //If end is reached
                    if (l.To == endNode)
                    {
                        bool getPath = true;
                        Node trackNode = endNode;
                        endNode.connectedFrom = l.From;
                        while (getPath)
                        {
                            path.Add(trackNode);
                            if (trackNode == startNode)
                                getPath = false;
                            else
                                trackNode = trackNode.connectedFrom;
                        }
                        searching = false;
                        break;
                    }

                    float reachDistance = calculateNodeDistance(n.stepCount + 1, l.To);
                    if (l.To.DistanceTracker == -1 || reachDistance < l.To.DistanceTracker)
                    {
                        l.To.stepCount = (n.stepCount + 1);
                        l.To.DistanceTracker = reachDistance;
                        l.To.connectedFrom = l.From;
                        leaves.Add(l.To);
                    }
                }

                leaves.Remove(n);
            }

            //Reset all pathfinding values
            clearNodes();

            //Algorithm returns path from end to start, so the list is reversed
            return reversePath(path);
        }

        private void clearNodes()
        {
            foreach (Node n in leaves)
            {
                bool reversing = true;
                Node trackNode = n;
                while (reversing)
                {
                    trackNode.stepCount = -1;
                    trackNode.DistanceTracker = -1;
                    Node temp = trackNode;

                    if (trackNode.connectedFrom != null)
                    {
                        trackNode = trackNode.connectedFrom;
                        temp.connectedFrom = null;
                    }
                    else
                        reversing = false;
                }
            }
            leaves.Clear();
        }

        //Debug function
        private void testNodes()
        {
            foreach(GridTracker grid in GenerateGrid.GetInstance().Grids)
            {
                foreach(Node n in grid.nodes)
                {
                    bool test = false;
                    if (n.connectedFrom != null)
                        test = true;
                    else if (n.DistanceTracker != -1)
                        test = true;
                    else if (n.stepCount != -1)
                        test = true;

                    if (test)
                    {
                        int k = 0;
                    }
                        
                }
            }
        }

        private List<Node> reversePath(List<Node> input)
        {
            List<Node> temp = new List<Node>();
            foreach (Node n in input)
                temp.Add(n);

            input.Clear();
            for (int i = temp.Count - 1; i >= 0; i--)
            {
                input.Add(temp[i]);
            }
            return input;
        }

        private float calculateNodeDistance(Node n)
        {
            return calculateNodeDistance(n.stepCount, n);
        }

        private float calculateNodeDistance(int i, Node n)
        {
            float val = Mathf.Pow(i, 2);
            val += getAxisDistance(n, endNode);

            return val;
        }

        private float getAxisDistance(Node n1, Node n2)
        {
            float x = Mathf.Abs(n1.posX - n2.posX);
            float z = Mathf.Abs(n1.posZ - n2.posZ);
            return (x + z);
        }

        private Node getShortestLeave()
        {
            float dis = -1;
            Node selectedNode = null;
            foreach (Node n in leaves)
            {
                if (n.DistanceTracker < dis || dis == -1)
                {
                    dis = n.DistanceTracker;
                    selectedNode = n;
                }
            }
            if (selectedNode == null)
            {
                int n = 0;
            }
            return selectedNode;
        }
    }
}
