﻿using System;
using System.Collections.Generic;
using UnityEngine;

namespace PathFinding
{
    public class NPCMovement : MonoBehaviour
    {
        public float MovementSpeed = 500f;

        private List<Node> currentPath;
        private Node lastTarget;
        private int _currTarget = 0;
        public bool Moving = false;
        private Vector3 _direction;
        private Animator _anim;
        private bool _doAnimations = true;
        private bool _immortal = false;
        private float animationSpeed = 2f;
        public Queue<List<Node>> Paths = new Queue<List<Node>>(); 
        public void InitPath(List<Node> n)
        {
            InitPath(true);
        }

        public void InitPath(bool m)
        {
            currentPath = Paths.Dequeue();
            transform.position = currentPath[0].Vector3(transform.position.y);
            _currTarget = 1;
            lastTarget = currentPath[0];
            Moving = m;
            if (currentPath != null)
                _direction = (currentPath[_currTarget].Vector3(0) - lastTarget.Vector3(0));
            if (_anim == null)
                _doAnimations = false;
        }

        public void GrantImmortality()
        {
            _immortal = true;
        }

        void Start()
        {
            if (GetComponent<Animator>() != null)
            {
                _anim = GetComponent<Animator>();
            }
            else
            {
                _anim = GetComponentInChildren<Animator>();
            }

        }

        public void StartMovement()
        {
            _direction = (currentPath[_currTarget].Vector3(0) - lastTarget.Vector3(0));
            transform.rotation = calculateRotationAngle(_direction);
            _anim.SetBool("walking", true);
            Moving = true;
        }

        void Update()
        {
            if (currentPath != null && currentPath.Count > 1)
            {
                if (reachedTarget())
                {
                    lastTarget = currentPath[_currTarget];
                    if (_currTarget < currentPath.Count - 1)
                    {
                        _currTarget++;
                        _direction = (currentPath[_currTarget].Vector3(0) - lastTarget.Vector3(0));
                        transform.rotation = calculateRotationAngle(_direction);
                    }

                    else if (!_immortal)
                        GameObject.FindGameObjectWithTag("NPC").GetComponent<NPCController>().KillNPC(this.gameObject);
                    else
                    {
                        if(Paths.Count > 0)
                        {
                            InitPath(true);
                        }
                        else
                        {
                            _direction = Vector3.zero;
                            _anim.SetBool("walking", false);
                        }
                    }
                }

                string whatever = transform.name;
                if (Moving) 
                    transform.position += _direction * MovementSpeed * Time.deltaTime;
            }
            
        }

        private bool reachedTarget()
        {
            if (Vector3.Distance(transform.position, currentPath[_currTarget].Vector3(transform.position.y)) < Vector3.Distance(transform.position, lastTarget.Vector3(transform.position.y)))
            {
                return true;
            }

            return false;
        }

        private Quaternion calculateRotationAngle(Vector3 dir)
        {
            Quaternion rotation = Quaternion.LookRotation(dir);

            return rotation;
        }

        void FixedUpdate()
        {
            //if (_doAnimations)
            //{
            //    if (!Moving && _anim != null)
            //        _anim.speed = 0;
            //    else if (_anim != null)
            //        _anim.speed = animationSpeed;
            //}
        }
    }
}
