﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Global;
using System;
using PathFinding;
using Assets._Scripts;
using Assets.Constants;

namespace Player
{
    public class PlayerMovement : MonoBehaviour, HasSituation
    {
        #region variables
        private GameObject _camera;
        private float _movementSpeed = 4.5f, _rotatedFrom, _lastTarget = -1, _cameraMaxRotationSpeed = 5f;
        private float verticalTranslation, horizontalTranslation;
        private Vector3 _cameraDisplacement;
        private GameObject _cameraController;
        public bool playerHasControl = true;
        private float _rotateSpeed = 1.5f;
        private float _rotation = 0f;
        private Situation mySituation;

        private static readonly float forwardDirection = 0f;
        private static readonly float rightDirection = 90f;
        private static readonly float backwardDirection = 180f;
        private static readonly float leftDirection = 270f;

        private static readonly float diff = 0.0001f;
        private static readonly float noRotation = -1;

        private float currentAngle = noRotation;
        private Animator anim;

        private NPCMovement movementScript;
        public bool FlipMovement = false;
        #endregion

        private void Awake()
        {
            movementScript = GetComponent<NPCMovement>();
            mySituation = new Situation()
            {
                Action = EndGame,
                Fires = true,
                Name = "EndGameEvent",
                SetOffTime = EventTimers.EndGameTime
            };
        }

        void Start()
        {
            _camera = GameObject.FindGameObjectWithTag("MainCamera");
            _cameraDisplacement = _camera.transform.position - transform.position;
            _cameraController = GameObject.FindGameObjectWithTag("Camera_Control");

            anim = GameObject.FindGameObjectWithTag("Player_Body").GetComponent<Animator>();
        }

        void Update()
        {
            verticalTranslation = 0;
            horizontalTranslation = 0;
            float mouse = Input.GetAxis("Mouse X");
            _rotation = mouse * _rotateSpeed;

            if (playerHasControl) {
                verticalTranslation = Input.GetAxis("Vertical") * _movementSpeed;
                horizontalTranslation = Input.GetAxis("Horizontal") * _movementSpeed;

                transform.position += _cameraController.transform.forward * verticalTranslation * Time.deltaTime;
                transform.position += _cameraController.transform.right * horizontalTranslation * Time.deltaTime;

                float newAngle = DetermineRotation(horizontalTranslation, verticalTranslation);
                if (!FloatEqual(currentAngle, newAngle))
                {
                    currentAngle = DetermineRotation(horizontalTranslation, verticalTranslation);
                    transform.eulerAngles = new Vector3(0, (_cameraController.transform.eulerAngles.y + DetermineRotation(verticalTranslation, horizontalTranslation)) % 360, 0);
                }
                transform.Rotate(new Vector3(0, _rotation));

                if (verticalTranslation > diff || verticalTranslation < -diff || horizontalTranslation > diff || horizontalTranslation < -diff)
                {
                    anim.SetBool("walking", true);
                } else
                {
                    anim.SetBool("walking", false);
                }
            }


            

            //transform.position = new Vector3(transform.position.x + horizontalTranslation * Time.deltaTime, transform.position.y, transform.position.z + verticalTranslation * Time.deltaTime);

        }

        private void LateUpdate()
        {
            _cameraController.transform.position = transform.position;
            _cameraController.transform.Rotate(new Vector3(0, _rotation));
        }

        public float DetermineRotation(float horizontal, float vertical)
        {
            float result = noRotation;
            float hor = noRotation;
            float ver = noRotation;

            //checking horizontal direction
            if (horizontal > diff)
            {
                hor = forwardDirection;
            }
            else if (horizontal < -diff)
            {
                hor = backwardDirection;
            }
            //checking vertical direction
            if (vertical > diff)
            {
                ver = rightDirection;
            }
            else if (vertical < -diff)
            {
                ver = leftDirection ;
            }

            if (!FloatEqual(hor, noRotation) && !FloatEqual(ver, noRotation))
            {
                result = (hor + ver) / 2;
                if(FloatEqual(ver, leftDirection) && FloatEqual(hor, forwardDirection))
                {
                    result = 270f + 22.5f;
                }
            }
            else if (!FloatEqual(hor, noRotation) || !FloatEqual(ver,noRotation))
            {
                result = !FloatEqual(hor, noRotation) ? hor : ver;
            }
            return result;
        }

        public bool FloatEqual(float a, float b)
        {
            bool result = true;
            if (Math.Abs(a - b) > diff)
            {
                result = false;
            }
            return result;
        }

        public Situation GetSituation()
        {
            return mySituation;
        }

        public void EndGame()
        {
            playerHasControl = false;
            GeneratePath myPath = new GeneratePath(transform.position, new Vector3(-3.983f, 0, 71f));
            movementScript.Paths.Enqueue(myPath.GetPath());
            movementScript.InitPath(true);
            movementScript.GrantImmortality();
            movementScript.StartMovement();
        }
    }
}
